@@ -1,3 +1,4 @@
+/* exported SearchProvider */
 // -*- Mode: js; indent-tabs-mode: nil; c-basic-offset: 4; tab-width: 4 -*-
 //
 // Copyright (c) 2013 Giovanni Campagna <scampa.giovanni@gmail.com>
@@ -17,10 +18,9 @@
 // with Gnome Weather; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-const {Gc, Gdk, Gio, GLib, GObject} = imports.gi;
+const { Gc, Gio, GLib, GObject } = imports.gi;
 
 const ByteArray = imports.byteArray;
-const Service = imports.service;
 const Util = imports.util;
 
 const MAX_SEARCH_RESULTS = 100;
@@ -51,12 +51,12 @@ var SearchProvider = GObject.registerClass({
 
         let upper = keywords.map(x => x.toUpperCase());
         let criteria = Gc.SearchCriteria.new_keywords(upper);
-        let context = new Gc.SearchContext({ criteria: criteria,
-                                             flags: Gc.SearchFlag.WORD });
+        let context = new Gc.SearchContext({ criteria, flags: Gc.SearchFlag.WORD });
+
         context.search(
             MAX_SEARCH_RESULTS,
             this._cancellable,
-            (source_object, res, user_data) => {
+            (_source, res, _userData) => {
                 let characters = [];
                 try {
                     let result = context.search_finish(res);
@@ -65,24 +65,18 @@ var SearchProvider = GObject.registerClass({
                     log(`Failed to search by keywords: ${e.message}`);
                 }
                 invocation.return_value(new GLib.Variant('(as)', [characters]));
-
-                this._app.release();
             });
     }
 
     GetInitialResultSetAsync(params, invocation) {
-        this._app.hold();
         this._runQuery(params[0], invocation);
     }
 
     GetSubsearchResultSetAsync(params, invocation) {
-        this._app.hold();
         this._runQuery(params[1], invocation);
     }
 
     GetResultMetas(identifiers) {
-        this._app.hold();
-
         let ret = [];
 
         for (let i = 0; i < identifiers.length; i++) {
@@ -90,69 +84,35 @@ var SearchProvider = GObject.registerClass({
             let codePoint = Util.toCodePoint(character);
             let codePointHex = codePoint.toString(16).toUpperCase();
             let name = Gc.character_name(character);
-            if (name == null)
-                name = _("Unknown character name");
+            if (name === null)
+                name = _('Unknown character name');
             else
                 name = Util.capitalize(name);
-            let summary = _("U+%s, %s: %s").format(codePointHex,
-                                                   character,
-                                                   name);
-            ret.push({ name: new GLib.Variant('s', name),
-                       id: new GLib.Variant('s', identifiers[i]),
-                       description: new GLib.Variant('s', summary),
-                       icon: (new Gio.ThemedIcon({ name: Service.application_id })).serialize(),
-                       clipboardText: new GLib.Variant('s', character)
-                     });
+            let summary = _('U+%s').format(codePointHex);
+
+            let iconData = Util.characterToIconData(character);
+            ret.push({
+                name: new GLib.Variant('s', name),
+                id: new GLib.Variant('s', identifiers[i]),
+                description: new GLib.Variant('s', summary),
+                clipboardText: new GLib.Variant('s', character),
+                'icon-data': iconData,
+            });
         }
-
-        this._app.release();
-
         return ret;
     }
 
-    ActivateResult(id, terms, timestamp) {
-        let clipboard = Gc.gtk_clipboard_get();
-        clipboard.set_text(id, -1);
-    }
-
-    _getPlatformData(timestamp) {
-        let display = Gdk.Display.get_default();
-        let context = display.get_app_launch_context();
-        context.set_timestamp(timestamp);
-
-        let app = Gio.DesktopAppInfo.new('org.gnome.Characters.desktop');
-        let id = context.get_startup_notify_id(app, []);
-        return {'desktop-startup-id': new GLib.Variant('s', id) };
-    }
-
-    _activateAction(action, parameter, timestamp) {
-        let wrappedParam;
-        if (parameter)
-            wrappedParam = [parameter];
-        else
-            wrappedParam = [];
-
-        Gio.DBus.session.call('org.gnome.Characters',
-                              '/org/gnome/Characters',
-                              'org.freedesktop.Application',
-                              'ActivateAction',
-                              new GLib.Variant('(sava{sv})', [action, wrappedParam,
-                                                              this._getPlatformData(timestamp)]),
-                              null,
-                              Gio.DBusCallFlags.NONE,
-                              -1, null, (connection, result) => {
-                                  try {
-                                      connection.call_finish(result);
-                                  } catch(e) {
-                                      log(`Failed to launch application: ${e.message}`);
-                                  }
-
-                                  this._app.release();
-                              });
+    ActivateResult(_id, _terms, _timestamp) {
+        log('activating result');
+        const notification = Gio.Notification.new(_('Character copied'));
+        notification.set_body(_('Character was copied successfully'));
+        this._app.send_notification(null, notification);
     }
 
     LaunchSearch(terms, timestamp) {
-        this._activateAction('search', new GLib.Variant('as', terms),
-                             timestamp);
+        this._app.activate();
+        const window = this._app.window;
+        window.setSearchKeywords(terms);
+        window.present_with_time(timestamp);
     }
 });
